@@ -7,6 +7,7 @@
 {-# LANGUAGE DeriveFunctor #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE ScopedTypeVariables #-}
 module Data.MinLen
     ( -- * Type level naturals
       -- ** Peano numbers
@@ -40,7 +41,7 @@ module Data.MinLen
     , minimumBy
     ) where
 
-import Prelude (Num (..), Maybe (..), Int, Ordering (..), Eq, Ord, Read, Show, Functor (..), ($), flip)
+import Prelude (Num (..), Maybe (..), Int, Ordering (..), Eq, Ord (..), Read, Show, Functor (..), ($), flip, const, Bool (..), otherwise)
 import Data.Data (Data)
 import Data.Typeable (Typeable)
 import Control.Category
@@ -50,6 +51,7 @@ import Data.Monoid (Monoid (..))
 import Data.Semigroup (Semigroup (..))
 import Data.GrowingAppend
 import Control.Monad (liftM)
+import Control.Monad.Trans.State.Strict (evalState, state)
 
 -- $peanoNumbers
 -- <https://wiki.haskell.org/Peano_numbers Peano numbers> are a simple way to
@@ -480,3 +482,32 @@ minimumBy :: MonoFoldable mono
           -> Element mono
 minimumBy cmp = minimumByEx cmp . unMinLen
 {-# INLINE minimumBy #-}
+
+-- | 'oextract' is 'head'.
+--
+-- For @'oextend' f@, the new 'mono' is populated by applying @f@ to
+-- successive 'tail's of the original 'mono'.
+--
+-- For example, for @'MinLen' ('Succ' 'Zero') ['Int']@, or
+-- @'NonNull' ['Int']@:
+--
+-- @
+-- 'oextend' f [1,2,3,4,5] = [ f [1, 2, 3, 4, 5]
+--                           , f [2, 3, 4, 5]
+--                           , f [3, 4, 5]
+--                           , f [4, 5]
+--                           , f [5]
+--                           ]
+-- @
+--
+-- Meant to be a direct analogy to the instance for 'NonEmpty' @a@.
+--
+instance IsSequence mono
+    => MonoComonad (MinLen (Succ Zero) mono) where
+        oextract  = head
+        oextend f (MinLen mono) = MinLen
+                                . flip evalState mono
+                                . ofor mono
+                                . const
+                                . state
+                                $ \mono' -> (f (MinLen mono'), tailEx mono')
